@@ -2,16 +2,18 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 from decimal import Decimal
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard, StateView, Button, StateAction
+
+from trytond.model import (ModelView, ModelSQL, MatchMixin, ValueMixin, fields,
+    sequence_ordered)
 from trytond import backend
-from trytond.pyson import Eval, PYSONEncoder, Date, If
+from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import grouped_slice
+from trytond.tools.multivalue import migrate_property
 
-__all__ = ['Location', 'Party', 'ProductsByLocationsStart',
-    'ProductsByLocations']
+__all__ = ['Location', 'Party', 'PartyLocation', 'ProductsByLocationsContext',
+    'LocationLeadTime']
 
 STATES = {
     'readonly': ~Eval('active'),
@@ -293,6 +295,7 @@ class Location(ModelSQL, ModelView):
         Set the parent of child location of warehouse if not set
         '''
         to_update = set()
+        to_save = []
         for location in locations:
             if location.type == 'warehouse':
                 if not location.input_location.parent:
@@ -302,10 +305,11 @@ class Location(ModelSQL, ModelView):
                 if not location.storage_location.parent:
                     to_update.add(location.storage_location)
                 if to_update:
-                    cls.write(list(to_update), {
-                        'parent': location.id,
-                        })
+                    for child_location in to_update:
+                        child_location.parent = location
+                        to_save.append(child_location)
                     to_update.clear()
+        cls.save(to_save)
 
     @classmethod
     def create(cls, vlist):
@@ -393,66 +397,127 @@ class Location(ModelSQL, ModelView):
         return res
 
 
+supplier_location = fields.Many2One(
+    'stock.location', "Supplier Location", domain=[('type', '=', 'supplier')],
+    help='The default source location when receiving products from the party.')
+customer_location = fields.Many2One(
+    'stock.location', "Customer Location", domain=[('type', '=', 'customer')],
+    help='The default destination location when sending products to the party.'
+    )
+
+
 class Party:
     __metaclass__ = PoolMeta
     __name__ = 'party.party'
-    supplier_location = fields.Property(fields.Many2One('stock.location',
-        'Supplier Location', domain=[('type', '=', 'supplier')],
-        help='The default source location when receiving products from the '
-        'party.'))
-    customer_location = fields.Property(fields.Many2One('stock.location',
-        'Customer Location', domain=[('type', '=', 'customer')],
-        help='The default destination location when sending products to the '
-        'party.'))
+    supplier_location = fields.MultiValue(supplier_location)
+    customer_location = fields.MultiValue(customer_location)
+    locations = fields.One2Many(
+        'party.party.location', 'party', "Locations")
+
+    @classmethod
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field in {'supplier_location', 'customer_location'}:
+            return pool.get('party.party.location')
+        return super(Party, cls).multivalue_model(field)
+
+    @classmethod
+    def default_supplier_location(cls, **pattern):
+        return cls.multivalue_model(
+            'supplier_location').default_supplier_location()
+
+    @classmethod
+    def default_customer_location(cls, **pattern):
+        return cls.multivalue_model(
+            'customer_location').default_customer_location()
 
 
-class ProductsByLocationsStart(ModelView):
+class PartyLocation(ModelSQL, ValueMixin):
+    "Party Location"
+    __name__ = 'party.party.location'
+    party = fields.Many2One(
+        'party.party', "Party", ondelete='CASCADE', select=True)
+    supplier_location = supplier_location
+    customer_location = customer_location
+
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        exist = TableHandler.table_exist(cls._table)
+
+        super(PartyLocation, cls).__register__(module_name)
+
+        if not exist:
+            cls._migrate_property([], [], [])
+
+    @classmethod
+    def _migrate_property(cls, field_names, value_names, fields):
+        field_names.extend(['supplier_location', 'customer_location'])
+        value_names.extend(['supplier_location', 'customer_location'])
+        migrate_property(
+            'party.party', field_names, cls, value_names,
+            parent='party', fields=fields)
+
+    @classmethod
+    def default_supplier_location(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id('stock', 'location_supplier')
+        except KeyError:
+            return None
+
+    @classmethod
+    def default_customer_location(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id('stock', 'location_customer')
+        except KeyError:
+            return None
+
+
+class ProductsByLocationsContext(ModelView):
     'Products by Locations'
-    __name__ = 'stock.products_by_locations.start'
+    __name__ = 'stock.products_by_locations.context'
     forecast_date = fields.Date(
         'At Date', help=('Allow to compute expected '
             'stock quantities for this date.\n'
             '* An empty value is an infinite date in the future.\n'
             '* A date in the past will provide historical values.'))
+    stock_date_end = fields.Function(fields.Date('At Date'),
+        'on_change_with_stock_date_end')
 
     @staticmethod
     def default_forecast_date():
         Date_ = Pool().get('ir.date')
         return Date_.today()
 
+    @fields.depends('forecast_date')
+    def on_change_with_stock_date_end(self, name=None):
+        if self.forecast_date is None:
+            return datetime.date.max
+        return self.forecast_date
 
-class ProductsByLocations(Wizard):
-    'Products by Locations'
-    __name__ = 'stock.products_by_locations'
-    start = StateView('stock.products_by_locations.start',
-        'stock.products_by_locations_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Open', 'open', 'tryton-ok', True),
+
+class LocationLeadTime(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
+    'Location Lead Time'
+    __name__ = 'stock.location.lead_time'
+
+    warehouse_from = fields.Many2One('stock.location', 'Warehouse From',
+        ondelete='CASCADE',
+        domain=[
+            ('type', '=', 'warehouse'),
             ])
-    open = StateAction('stock.act_products_by_locations')
+    warehouse_to = fields.Many2One('stock.location', 'Warehouse To',
+        ondelete='CASCADE',
+        domain=[
+            ('type', '=', 'warehouse'),
+            ])
+    lead_time = fields.TimeDelta('Lead Time')
 
-    def do_open(self, action):
-        pool = Pool()
-        Location = pool.get('stock.location')
-        Lang = pool.get('ir.lang')
-
-        context = {}
-        context['locations'] = Transaction().context.get('active_ids')
-        date = self.start.forecast_date or datetime.date.max
-        context['stock_date_end'] = Date(date.year, date.month, date.day)
-        action['pyson_context'] = PYSONEncoder().encode(context)
-
-        locations = Location.browse(context['locations'])
-
-        for code in [Transaction().language, 'en_US']:
-            langs = Lang.search([
-                    ('code', '=', code),
-                    ])
-            if langs:
-                break
-        lang = langs[0]
-        date = Lang.strftime(date, lang.code, lang.date)
-
-        action['name'] += ' - (%s) @ %s' % (
-            ','.join(l.name for l in locations), date)
-        return action, {}
+    @classmethod
+    def get_lead_time(cls, pattern):
+        for record in cls.search([]):
+            if record.match(pattern):
+                return record.lead_time
