@@ -728,6 +728,165 @@ class StockTestCase(ModuleTestCase):
                 states[state].sort()
             self.assertEqual(states, result, msg=msg)
 
+    @with_transaction()
+    def test_assign_try_chained(self):
+        "Test Move assign_try chained"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+
+        uom, = Uom.search([('name', '=', 'Meter')])
+        template = Template(
+            name='Test Move.assign_try',
+            type='goods',
+            list_price=Decimal(1),
+            cost_price=Decimal(0),
+            cost_price_method='fixed',
+            default_uom=uom,
+            )
+        template.save()
+        product = Product(template=template.id)
+        product.save()
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        storage2, = Location.copy([storage])
+        storage3, = Location.copy([storage])
+
+        company = create_company()
+        with set_company(company):
+            move, = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+            Move.do([move])
+
+            moves = Move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': from_.id,
+                        'to_location': to.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        } for from_, to in [
+                        (storage, storage2),
+                        (storage2, storage3)]])
+
+            self.assertFalse(Move.assign_try(moves))
+            self.assertEqual([m.state for m in moves], ['assigned', 'draft'])
+
+    @with_transaction()
+    def test_assign_without_moves(self):
+        "Test Move assign_try with empty moves"
+        pool = Pool()
+        Move = pool.get('stock.move')
+
+        self.assertTrue(Move.assign_try([]))
+
+    @with_transaction()
+    def test_products_by_location_assign(self):
+        "Test products by location for assignation"
+        pool = Pool()
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Uom = pool.get('product.uom')
+        Location = pool.get('stock.location')
+        Move = pool.get('stock.move')
+        today = datetime.date.today()
+
+        unit, = Uom.search([('name', '=', 'Unit')])
+        template = Template(
+            name="Product",
+            type='goods',
+            list_price=Decimal(1),
+            cost_price=Decimal(1),
+            cost_price_method='fixed',
+            default_uom=unit,
+            )
+        template.save()
+        product, = Product.create([{'template': template.id}])
+
+        supplier, = Location.search([('code', '=', 'SUP')])
+        storage, = Location.search([('code', '=', 'STO')])
+        customer, = Location.search([('code', '=', 'CUS')])
+
+        company = create_company()
+        with set_company(company):
+            move_supplier, = Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 10,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'planned_date': today,
+                        'company': company.id,
+                        'unit_price': product.cost_price,
+                        'currency': company.currency.id,
+                        }])
+            move_customer, = Move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 5,
+                        'from_location': storage.id,
+                        'to_location': customer.id,
+                        'planned_date': today + relativedelta(days=1),
+                        'company': company.id,
+                        'unit_price': product.list_price,
+                        'currency': company.currency.id,
+                        }])
+
+            Move.assign([move_supplier])
+            with Transaction().set_context(
+                    stock_date_end=today,
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {})
+
+            Move.assign([move_customer])
+            with Transaction().set_context(
+                    stock_date_end=today,
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {(storage.id, product.id): -5})
+
+            Move.do([move_supplier])
+            with Transaction().set_context(
+                    stock_date_end=today,
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {(storage.id, product.id): 5})
+
+            with Transaction().set_context(
+                    stock_date_end=today + relativedelta(days=1),
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {(storage.id, product.id): 5})
+
+            with Transaction().set_context(
+                    stock_date_start=today,
+                    stock_date_end=today,
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {(storage.id, product.id): 10})
+
+            with Transaction().set_context(
+                    stock_date_start=today + relativedelta(days=1),
+                    stock_date_end=today + relativedelta(days=1),
+                    stock_assign=True):
+                pbl = Product.products_by_location([storage.id], [product.id])
+                self.assertDictEqual(pbl, {(storage.id, product.id): -5})
+
 
 def suite():
     suite = trytond.tests.test_tryton.suite()
@@ -736,6 +895,11 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_shipment_out_same_storage_output.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite(
             'scenario_stock_average_cost_price.rst',
             tearDown=doctest_teardown, encoding='utf-8',
